/*
 * Wine Conformance Test EXE
 *
 * Copyright 2003 Jakob Eriksson   (for Solid Form Sweden AB)
 * Copyright 2003 Dimitrie O. Paun
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 *
 * This program is dedicated to Anna Lindh,
 * Swedish Minister of Foreign Affairs.
 * Anna was murdered September 11, 2003.
 *
 */

#ifdef WINELIB
#  include "config.h"
#  include "wine/port.h"
#  define mkdir(d) mkdir (d, 0777)
#  include <errno.h>
#else
#  include <stdlib.h>
#  include <unistd.h>
#endif

#include <stdio.h>
#include <wtypes.h>

#include "winetests.h"
#include "send.h"
#include "util.h"

void print_version ()
{
    OSVERSIONINFOEX ver;
    BOOL ext;

    ver.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
    if (!(ext = GetVersionEx ((OSVERSIONINFO *) &ver)))
    {
	ver.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if (!GetVersionEx ((OSVERSIONINFO *) &ver))
	    fatal("Can't get OS version.");
    }

    xprintf ("    dwMajorVersion=%ld\n    dwMinorVersion=%ld\n"
             "    dwBuildNumber=%ld\n    PlatformId=%ld\n    szCSDVersion=%s\n",
             ver.dwMajorVersion, ver.dwMinorVersion, ver.dwBuildNumber,
             ver.dwPlatformId, ver.szCSDVersion);

    if (ext)
        xprintf ("    wServicePackMajor=%d\n    wServicePackMinor=%d\n"
                 "    wSuiteMask=%d\n    wProductType=%d\n    wReserved=%d\n",
                 ver.wServicePackMajor, ver.wServicePackMinor, ver.wSuiteMask,
                 ver.wProductType, ver.wReserved);
}

#define IsDotDir(x)     ((x[0] == '.') && ((x[1] == 0) || ((x[1] == '.') && (x[2] == 0))))

void removeDir (const char *dir)
{
  HANDLE  hFind;
  WIN32_FIND_DATAA wfd;
  char path[MAX_PATH];
  size_t dirlen = strlen (dir);

  /* Make sure the directory exists before going further */
  memcpy (path, dir, dirlen);
  strcpy (path + dirlen++, "\\*");
  hFind = FindFirstFileA (path, &wfd);
  if (hFind == INVALID_HANDLE_VALUE)
    return;

  do {
    char *lp = wfd.cFileName;

    if (!lp[0]) lp = wfd.cAlternateFileName; /* ? FIXME not (!lp) ? */
    if (IsDotDir (lp)) continue;
    strcpy (path + dirlen, lp);
    if (FILE_ATTRIBUTE_DIRECTORY & wfd.dwFileAttributes)
        removeDir(path);
    else if (!DeleteFileA (path))
             fatal (strmake (NULL, "Can't delete file %s: error %d", path, GetLastError ()));
  } while (FindNextFileA (hFind, &wfd));
  FindClose (hFind);
  if (!RemoveDirectoryA (dir))
      fatal (strmake (NULL, "Can't remove directory %s: error %d", dir, GetLastError ()));
}

int count_tests()
{
    struct wine_test* test;
    int total = 0;

    for (test = wine_tests; test->name; test++)
	total += test->subtest_count;

    return total;
}

LPVOID extract_rcdata (const char *name, DWORD* size)
{
    HRSRC rsrc;
    HGLOBAL hdl;

    rsrc = FindResource (0, name, "USERDATA");
    if (!rsrc) return 0;
    *size = SizeofResource (0, rsrc);
    if (!*size) return 0;
    hdl = LoadResource (0, rsrc);
    if (!hdl) return 0;
    return LockResource (hdl);
}

void extract_test (const char *dir, struct wine_test* test)
{
    BYTE* code;
    DWORD size;
    FILE* fout;

    code = extract_rcdata (test->resource, &size);
    if (!code) fatal (strmake (NULL, "Can't get resource %s.", test->resource));

    test->is_elf = (code[1] == 'E' && code[2] == 'L' && code[3] == 'F');
    test->exename = strmake(NULL, "%s/%s_crosstest.exe%s", dir, test->name,
			    test->is_elf ? ".so" : "");

    if (!(fout = fopen(test->exename, "wb")) ||
        (fwrite (code, size, 1, fout) != 1) ||
        fclose (fout)) fatal (strmake (NULL, "Failed to write file %s.", test->name));
}

void run_test (struct wine_test* test, const char* subtest)
{
    int status;
    const char *argv[]={"wine", test->exename, subtest, NULL};

    xprintf ("%s:%s start\n", test->name, subtest);
    if (test->is_elf)
      status = spawnvp (_P_WAIT, "/usr/local/src/wine/wine", argv);
    else
      status = spawnvp (_P_WAIT, test->exename, argv+1);
    if (status)
        xprintf ("Exit status: %d, errno=%d: %s\n", status, errno, strerror (errno));
    xprintf ("%s:%s done\n", test->name, subtest);
}

int WINAPI WinMain (HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR cmdline, int cmdshow)
{
    struct wine_test* test;
    int nr_of_tests, subtest;
    char *tempdir, *logfile;
    FILE *fp;

    SetErrorMode (SEM_FAILCRITICALERRORS);

    if (setvbuf (stdout, NULL, _IONBF, 0)) fatal ("Can't unbuffer output.");

    tempdir = tempnam (0, "wct");
    if (!tempdir) fatal ("Can't name temporary dir (check TMP).");
    fprintf (stderr, "tempdir=%s\n", tempdir);
    if (mkdir (tempdir)) fatal (strmake (NULL, "Could not create directory: %s", tempdir));

    logfile = tempnam (0, "res");
    if (!logfile) fatal ("Can't name log file.");
    fprintf (stderr, "logfile=%s\n", logfile);

    fp = fopen (logfile, "ab");
    if (!fp) fatal ("Could not open log file.");
    if (-1 == dup2 (fileno (fp), 1)) fatal ("Can't redirect stdout.");

    xprintf ("Tests from build %s\n", build_tag);
    xprintf ("Operating system version:\n");
    print_version ();
    xprintf ("Test output:\n" );

    for (test = wine_tests; test->name; test++)
        extract_test (tempdir, test);

    nr_of_tests = count_tests();

    for (test = wine_tests; test->name; test++)
	for (subtest = 0; subtest < test->subtest_count; subtest++)
	    run_test (test, test->subtests[subtest]);

    fclose (fp);
    close (1);                  /* Avoid sharing violation on Win */

    removeDir (tempdir);

    if (send_file (logfile))
      fatal ("Can't submit logfile (network of file error).");

    if (remove (logfile))
      fatal (strmake (NULL, "Can't remove logfile: %d.", errno));

    return 0;
}
